package core;

import core.GameServer.Spectator;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamClass;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import network.Message;
import network.MessageFactory;
import tools.NullStream;

/**
 * Logs messages that where broadcasted to all players.
 *
 * @author ubik
 */
public class GameServerLog {

    
    private ObjectOutputStream log = NullStream.out(ObjectOutputStream.class);
    private final ArrayList<Message> messageLog = new ArrayList<Message>();

    public void setLogFile(String path) {
        try {
            File file = new File(path);
            if (!file.exists()) {
                file.createNewFile();
            }

            if (!file.exists()) {
                Logger.getLogger(GameServerLog.class.getName()).log(Level.SEVERE, null, "Could not create log file at " + path);
            } else {
                log = new ObjectOutputStream(new FileOutputStream(file));
            }
        } catch (IOException ex) {
            Logger.getLogger(GameServerLog.class.getName()).log(Level.SEVERE, null, ex);
        }

        // Write backlog
        try {
            for (Message message : messageLog) {
                log.writeObject(message);
            }
        } catch (IOException ex) {
            Logger.getLogger(GameServerLog.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void logMessage(Message message) {
        message = MessageFactory.copy(message);
        synchronized (messageLog) {
            messageLog.add(message);
            try {
                log.writeObject(message);
            } catch (IOException ex) {
                Logger.getLogger(GameServerLog.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void replay(Spectator spectator) {
        synchronized (messageLog) {
            DataOutputStream stream = null;
            try {
                stream = spectator.getStream();
                for (Message message : messageLog) {
                    message.write(stream);
                    stream.flush();
                }
            } catch (IOException ex) {
                Logger.getLogger(GameServerLog.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    

}
