package core;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

import static network.NetworkConstants.MSG_HELLO;
import static network.NetworkConstants.MSG_BYE;

/**
 *
 * @author reidl
 */
public class KnockServer implements Runnable {

    private GameServer server;
    private int port;
    private ServerSocket sock;

    private KnockServer(GameServer server, int port) {
        this.server = server;
        this.port = port;
    }
    
    private static void log(Object msg) {
        Logger logger = Logger.getLogger("Server");
        logger.log(Level.INFO, "Knock Server: {0}", msg.toString());
    }
    
    @Override
    public void run() {
        try {
            sock = new ServerSocket(port);
        } catch (IOException ex) {
            Logger.getLogger(KnockServer.class.getName()).log(Level.SEVERE, null, ex);
            return;
        }

        while (true) {
            log(" Waiting for client on port " + sock.getLocalPort());
            // Wait for client
            Socket clientSock = null;
            try {
                clientSock = sock.accept();
                clientSock.setSoTimeout(50);

                log(" Client accepted.");
                DataOutputStream out = new DataOutputStream(clientSock.getOutputStream());

                ServerSocket privateSock = new ServerSocket(0); // Takes next free port
                int clientPort = privateSock.getLocalPort();
                assert (clientPort != -1);

                log(" Writing hello and suggesting port " + clientPort);
                out.writeByte(MSG_HELLO);
                out.writeInt(clientPort);
                out.flush();

                log(" Waiting for BYE");

                try {
                    DataInputStream in = new DataInputStream(clientSock.getInputStream());
                    byte answer = in.readByte();
                    if (answer == MSG_BYE) {
                        int len = in.readInt();
                        StringBuilder name = new StringBuilder();
                        for (int i = 0; i < len; i++) {
                            name.append(in.readChar());
                        }

                        if (len != 0) {
                            server.acceptClient(name.toString(), privateSock);
                        } else {
                            server.acceptSpectator(privateSock);
                        }
                    }
                } catch (SocketTimeoutException ex) {
                    log("Client timeout.");
                }
                clientSock.close();
            } catch (IOException ex) {
                Logger.getLogger(KnockServer.class.getName()).log(Level.SEVERE, null, ex);
                return;
            }
        }
    }

    public static KnockServer create(GameServer callback, int port) {
        KnockServer res = new KnockServer(callback, port);
        log("Starting.");
        (new Thread(res)).start();
        return res;
    }
}
