package setup;

import core.LufBoard;
import core.Position;
import core.TileType;

/**
 * Paints a pattern to make debugging easier.
 *
 * @author reidl
 */
public class DebugBoardBuilder extends ProceduralBoardBuilder {

    @Override
    protected LufBoard buildInternal(LufBoard board) {
        TileType[][] matrix = new TileType[width][height];
        buildRec(matrix, 0, 0, width - 1, height - 1);

        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                assert matrix[x][y] != null : x +", " + y + " not set";
            }
        }

        Position[] playerPositions = new Position[numPlayers];
        for (int i = 0; i < playerPositions.length; i++) {
            playerPositions[i] = Position.create((width-1)*(i%2), (height-1)*((i%4)/2));
        }

        numObjectives = numPlayers*4;
        
        Position[] objectivePositions = new Position[numPlayers];
        for (int i = 0; i < objectivePositions.length; i++) {
            objectivePositions[i] = Position.create(width/2, height/2);
        }
                
        board.setBoard(matrix, TileType.LTRB, objectivePositions, playerPositions, new Position[]{Position.create(width / 2, height / 2)});

        return board;
    }

    private void buildRec(TileType[][] res, int left, int top, int right, int bottom) {
        if (right - left <= 0 || bottom - top <= 0) {
            if( res[left][top] == null )
                res[left][top] = TileType.BLANK;
            return;
        }

        int midx = (left + right) / 2;
        int midy = (top + bottom) / 2;

        for (int x = left + 1; x <= right - 1; x++) {
            res[x][top] = TileType.LR;
            res[x][midy] = TileType.LR;
            res[x][bottom] = TileType.LR;
        }

        for (int y = top + 1; y <= bottom - 1; y++) {
            res[left][y] = TileType.TB;
            res[midx][y] = TileType.TB;
            res[right][y] = TileType.TB;
        }

        // Corners
        res[left][top] = TileType.RB;
        res[right][top] = TileType.LB;
        res[left][bottom] = TileType.TR;
        res[right][bottom] = TileType.LT;

        // Ts
        if (bottom - top >= 2) {
            res[left][midy] = TileType.TRB;
            res[right][midy] = TileType.LTB;
        }

        if (right - left >= 2) {
            res[midx][top] = TileType.LRB;
            res[midx][bottom] = TileType.LTR;
        }

        // Mid-cross
        if (right - left >= 2 && bottom - top >= 2) {
            res[midx][midy] = TileType.LTRB;
        }

        buildRec(res, left + 1, top + 1, midx - 1, midy - 1);
        buildRec(res, midx + 1, top + 1, right - 1, midy - 1);
        buildRec(res, left + 1, midy + 1, midx - 1, bottom - 1);
        buildRec(res, midx + 1, midy + 1, right - 1, bottom - 1);
    }
}
