package setup;

import core.LufBoard;
import core.LufBoardEvent;
import core.LufBoardEvent.Visitor;
import java.util.LinkedList;
import java.util.List;

/**

 @author reidl
 */
public abstract class ProceduralBoardBuilder extends BoardBuilder {

    protected int width, height, numObjectives;

    public ProceduralBoardBuilder() {
        width = height = 14; // Arbitrary
        numPlayers = 0;
        numObjectives = 0;
    }

    public ProceduralBoardBuilder setHeight(int height) {
        assert height >= 2 : " Height most be at least two, is " + height;
        this.height = height;
        return this;
    }

    public ProceduralBoardBuilder setWidth(int width) {
        assert width >= 2 : " Width most at least two, is " + width;
        this.width = width;
        return this;
    }

    public ProceduralBoardBuilder setNumObjectives(int numObjectives) {
        this.numObjectives = numObjectives;
        return this;
    }

    @Override
    public ProceduralBoardBuilder addVisitor(Visitor visitor) {
        super.addVisitor(visitor);
        return this;
    }

    @Override
    public ProceduralBoardBuilder setNumPlayers(int numPlayers) {
        super.setNumPlayers(numPlayers);
        return this;
    }

    final public LufBoard build() {
        assert(width > 0 && height > 0);
        LufBoard res = new LufBoard(width, height);
        addVisitorsToBoard(res);
        return buildInternal(res);
    }

    protected abstract LufBoard buildInternal(LufBoard board);
}
